# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Shared flags for Cloud Workflows commands."""


from googlecloudsdk.api_lib.util import apis
from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import exceptions
from googlecloudsdk.calliope.concepts import concepts
from googlecloudsdk.calliope.concepts import deps
from googlecloudsdk.command_lib.util.args import labels_util
from googlecloudsdk.command_lib.util.args import map_util
from googlecloudsdk.command_lib.util.concepts import concept_parsers
from googlecloudsdk.core import properties
from googlecloudsdk.core.util import files
import six

_KEY_NAME_PATTERN = (
    r'^projects/[^/]+/locations/[^/]+/keyRings/[a-zA-Z0-9_-]+'
    '/cryptoKeys/[a-zA-Z0-9_-]+$'
)
_KEY_NAME_ERROR = (
    'KMS key name should match projects/{project}/locations/{location}'
    '/keyRings/{keyring}/cryptoKeys/{cryptokey} and only contain characters '
    'from the valid character set for a KMS key.'
)
USER_ENV_VARS_LIMIT = 20
CLEAR_ENVIRONMENT = object()


def LocationAttributeConfig():
  """Builds an AttributeConfig for the location resource."""
  return concepts.ResourceParameterAttributeConfig(
      name='location',
      fallthroughs=[
          deps.PropertyFallthrough(properties.FromString('workflows/location'))
      ],
      help_text=(
          'Cloud location for the {resource}. '
          ' Alternatively, set the property [workflows/location].'
      ),
  )


def WorkflowAttributeConfig():
  """Builds an AttributeConfig for the workflow resource."""
  return concepts.ResourceParameterAttributeConfig(
      name='workflow', help_text='Workflow for the {resource}.'
  )


def ExecutionAttributeConfig():
  """Builds an AttributeConfig for the execution resource."""
  return concepts.ResourceParameterAttributeConfig(
      name='execution', help_text='Execution for the {resource}.'
  )


def GetWorkflowResourceSpec():
  """Builds a ResourceSpec for the workflow resource."""
  return concepts.ResourceSpec(
      'workflows.projects.locations.workflows',
      resource_name='workflow',
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      locationsId=LocationAttributeConfig(),
      workflowsId=WorkflowAttributeConfig(),
  )


def GetExecutionResourceSpec():
  """Builds a ResourceSpec for the execution resource."""
  return concepts.ResourceSpec(
      'workflowexecutions.projects.locations.workflows.executions',
      resource_name='execution',
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      workflowsId=WorkflowAttributeConfig(),
      locationsId=LocationAttributeConfig(),
      executionsId=ExecutionAttributeConfig(),
  )


def AddWorkflowResourceArg(parser, verb):
  """Add a resource argument for a Cloud Workflows workflow.

  Args:
    parser: the parser for the command.
    verb: str, the verb to describe the resource, such as 'to update'.
  """
  concept_parsers.ConceptParser.ForResource(
      'workflow',
      GetWorkflowResourceSpec(),
      'Name of the workflow {}.'.format(verb),
      required=True,
  ).AddToParser(parser)


def AddExecutionResourceArg(parser, verb):
  """Add a resource argument for a Cloud Workflows execution.

  Args:
    parser: the parser for the command.
    verb: str, the verb to describe the resource, such as 'to update'.
  """
  concept_parsers.ConceptParser.ForResource(
      'execution',
      GetExecutionResourceSpec(),
      'Name of the execution {}.'.format(verb),
      required=True,
  ).AddToParser(parser)


def AddSourceArg(parser):
  """Adds argument for specifying source for the workflow."""
  parser.add_argument(
      '--source',
      help=(
          'Location of a workflow source code to deploy. Required on first '
          'deployment. Location needs to be defined as a path to a local file '
          'with the source code.'
      ),
  )


def AddDescriptionArg(parser):
  """Adds argument for specifying description of the workflow."""
  parser.add_argument(
      '--description', help='The description of the workflow to deploy.'
  )


def AddServiceAccountArg(parser):
  """Adds argument for specifying service account used by the workflow."""
  parser.add_argument(
      '--service-account',
      help=(
          'The service account that should be used as the workflow identity.'
          ' "projects/PROJECT_ID/serviceAccounts/" prefix may be skipped from'
          ' the full resource name, in that case "projects/-/serviceAccounts/"'
          ' is prepended to the service account ID.'
      ),
  )


def AddDataArg(parser):
  """Adds argument for specifying the data that will be passed to the workflow."""
  parser.add_argument(
      '--data',
      help=(
          'JSON string with data that will be passed to the workflow '
          'as an argument.'
      ),
  )


def AddLoggingArg(parser):
  """Adds argument for specifying the logging level for an execution."""
  log_level = base.ChoiceArgument(
      '--call-log-level',
      choices={
          'none': 'No logging level specified.',
          'log-all-calls': (
              'Log all calls to subworkflows or library functions and their'
              ' results.'
          ),
          'log-errors-only': 'Log when a call is stopped due to an exception.',
          'log-none': 'Perform no call logging.',
      },
      help_str='Level of call logging to apply during execution.',
      default='none',
  )
  log_level.AddToParser(parser)


def AddExecutionHistoryLevelArg(parser):
  """Adds argument for specifying the execution history level for an execution."""
  execution_history_level = base.ChoiceArgument(
      '--execution-history-level',
      choices={
          'none': 'No execution history level specified.',
          'execution-history-basic': (
              'Enable execution history basic feature.'
          ),
          'execution-history-detailed': (
              'Enable execution history detailed feature.'
          ),
      },
      help_str='Level of execution history to apply during execution.',
      default='none',
  )
  execution_history_level.AddToParser(parser)


def AddDisableOverflowBufferArg(parser):
  """Adds an argument for determining whether to backlog the execution."""
  parser.add_argument(
      '--disable-concurrency-quota-overflow-buffering',
      action='store_true',
      default=False,
      help=(
          'If set, the execution will not be backlogged when the concurrency '
          'quota is exhausted. Backlogged executions start when the '
          'concurrency quota becomes available.'
      ),
  )


def AddBetaLoggingArg(parser):
  """Adds argument for specifying the logging level for an execution."""
  log_level = base.ChoiceArgument(
      '--call-log-level',
      choices={
          'none': 'Perform no call logging.',
          'log-all-calls': (
              'Log all calls to subworkflows or library functions and their'
              ' results.'
          ),
          'log-errors-only': 'Log when a call is stopped due to an exception.',
      },
      help_str='Level of call logging to apply during execution.',
      default='none',
  )
  log_level.AddToParser(parser)


def AddWorkflowLoggingArg(parser):
  """Adds argument for specifying the logging level for a workflow."""
  log_level = base.ChoiceArgument(
      '--call-log-level',
      choices={
          'none': 'No logging level specified.',
          'log-all-calls': (
              'Log all calls to subworkflows or library functions and their'
              ' results.'
          ),
          'log-errors-only': 'Log when a call is stopped due to an exception.',
          'log-none': 'Perform no call logging.',
      },
      help_str='Level of call logging to apply by default for the workflow.',
      default='none',
  )
  log_level.AddToParser(parser)


def AddWorkflowTagsArg(parser):
  """Adds argument for specifying the tags for a workflow."""
  help_parts = [
      'List of tags KEY=VALUE pairs to bind.',
      'Each item must be expressed as',
      '"<tag-key-namespaced-name>=<tag-value-short-name>".\n',
      'Example: 123/environment=production,123/costCenter=marketing',
  ]
  tags = base.Argument(
      '--tags',
      metavar='KEY=VALUE',
      type=arg_parsers.ArgDict(),
      action=arg_parsers.UpdateAction,
      help='\n'.join(help_parts),
  )
  tags.AddToParser(parser)


def SetWorkflowsTagsArg(args, workflow, tags_message):
  """Sets --tags for the workflow based on the arguments."""
  if args.IsSpecified('tags'):
    tags = getattr(args, 'tags')
    if not tags:
      return None
    # Sorted for test stability
    workflow.tags = tags_message(additionalProperties=[
        tags_message.AdditionalProperty(key=key, value=value)
        for key, value in sorted(six.iteritems(tags))])


def AddWorkflowExecutionHistoryLevelArg(parser):
  """"Adds argument for specifying the execution history level for a workflow."""
  execution_history_level = base.ChoiceArgument(
      '--execution-history-level',
      choices={
          'none': 'No execution history level specified.',
          'execution-history-basic': (
              'Enable basic execution history.'
          ),
          'execution-history-detailed': (
              'Enable detailed execution history, including expected'
              ' iterations and in-scope variable values.'
          ),
      },
      help_str='Level of execution history to apply for the workflow.',
      default='none',
  )
  execution_history_level.AddToParser(parser)


def SetWorkflowLoggingArg(loglevel, workflow, updated_fields):
  """Sets --call-log-level for the workflow based on the arguments.

  Also updates updated_fields accordingly.

  Args:
    loglevel: Parsed callLogLevel to be set on the workflow.
    workflow: The workflow in which to set the call-log-level.
    updated_fields: A list to which the call-log-level field will be added if
      needed.
  """
  if loglevel is not None:
    workflow.callLogLevel = loglevel
    updated_fields.append('callLogLevel')


def SetWorkflowExecutionHistoryLevelArg(
    execution_history_level, workflow, updated_fields):
  """Sets --execution-history-level for the workflow based on the arguments.

  Also updates updated_fields accordingly.

  Args:
    execution_history_level: Parsed executionHistoryLevel to be set
    on the workflow.
    workflow: The workflow in which to set the execution-history-level.
    updated_fields: A list to which the execution-history-level field will
    be added if needed.
  """
  if execution_history_level is not None:
    workflow.executionHistoryLevel = execution_history_level
    updated_fields.append('executionHistoryLevel')


# Flags for CMEK
def AddKmsKeyFlags(parser):
  """Adds flags for configuring the CMEK key.

  Args:
    parser: The flag parser used for the specified command.
  """
  kmskey_group = parser.add_group(mutex=True, hidden=True)
  kmskey_group.add_argument(
      '--kms-key',
      type=arg_parsers.RegexpValidator(_KEY_NAME_PATTERN, _KEY_NAME_ERROR),
      help="""\
        Sets the user managed KMS crypto key used to encrypt the new Workflow
        Revision and the Executions associated with it.

        The KMS crypto key name should match the pattern
        `projects/${PROJECT}/locations/${LOCATION}/keyRings/${KEYRING}/cryptoKeys/${CRYPTOKEY}`
        where ${PROJECT} is the project, ${LOCATION} is the location of the key
        ring, and ${KEYRING} is the key ring that contains the ${CRYPTOKEY}
        crypto key.
      """,
  )
  kmskey_group.add_argument(
      '--clear-kms-key',
      action='store_true',
      help="""\
        Creates the new Workflow Revision and its associated Executions without
        the KMS key specified on the previous revision.
      """,
  )


def SetKmsKey(args, workflow, updated_fields):
  """Sets KMS key for the workflow based on the arguments.

  Also update updated_fields accordingly.

  Args:
    args: Args passed to the command.
    workflow: The workflow in which to set the KMS key.
    updated_fields: A list to which the KMS key field will be added if needed.
  """
  if args.IsSpecified('kms_key') or args.IsSpecified('clear_kms_key'):
    workflow.cryptoKeyName = None if args.clear_kms_key else args.kms_key
    updated_fields.append('cryptoKeyName')


def AddUserEnvVarsFlags(parser):
  """Adds flags for configuring user-defined environment variables."""
  userenvvars_group = parser.add_group(
      mutex=True,
      hidden=False,
      help="""\
        Flags to configure user-defined environment variables for a
        workflow.

        Keys can't be empty strings and can't start with `GOOGLE`
        or `WORKFLOWS`. We recommend that environment variable keys consist
        solely of uppercase letters, digits, and underscores (`_`), and that
        they don't begin with a digit. Consider prefixing your user-defined
        environment variables with a unique key to avoid conflicts with other
        variables.

        If your value contains commas, prefix the mapping with a different
        delimiter character enclosed between `^` (example 1). Use special
        characters in your shell with caution as they might not work as
        intended or need escaping (example 2 escapes a `$` in Bash).

        Example 1: --set-env-vars ^@^KEY1=ONE,VALUE,WITH,COMMAS@KEY2=VALUE2

        Example 2: --set-env-vars ^$^KEY1=VALUE1\\$KEY2=VALUE,WITH,COMMAS,TOO

        A maximum of 20 user-defined environment variables can be defined.
        Each definition string (`KEY=value`) is limited to 4 KiB.
        All keys and values are converted to strings.
      """,
  )

  userenvvars_group.add_argument(
      '--set-env-vars',
      type=arg_parsers.ArgDict(
          key_type=str,
          value_type=str,
          max_length=USER_ENV_VARS_LIMIT,
      ),
      action=arg_parsers.UpdateAction,
      metavar='KEY=VALUE',
      help="""\
        Sets environment variables for the workflow based on a comma-separated
        list of key-value pairs. Will overwrite a workflow's existing
        environment variables.

        Example:
        gcloud workflows deploy ${workflow_name} --set-env-vars policy=global,retry_count=5
      """,
  )

  userenvvars_group.add_argument(
      '--env-vars-file',
      metavar='FILE_PATH',
      type=map_util.ArgDictFile(key_type=str, value_type=str),
      help="""\
        Sets environment variables for the workflow to those stored in a local
        YAML file at the given path. All existing environment variables are
        removed before the new environment variables are added.

        Example:
        gcloud workflows deploy ${workflow_name} --env-vars-file=/path/to/env_vars.yaml

        Inside env_vars.yaml:\n
        policy: global\n
        retry_count: 5 # Service converts this to string "5"
      """,
  )

  userenvvars_group.add_argument(
      '--clear-env-vars',
      action='store_true',
      help="""\
        Clears all user-defined environment variables previously set for the
        workflow.

        Example:
        gcloud workflows deploy ${workflow_name} --clear-env-vars
      """,
  )

  userenvvars_group.add_argument(
      '--remove-env-vars',
      metavar='KEY',
      action=arg_parsers.UpdateAction,
      type=arg_parsers.ArgList(element_type=str),
      help="""\
        Removes user-defined environment variables from a workflow based
        on a list of environment variable keys to be removed.

        Example:
        gcloud workflows deploy ${workflow_name} --remove-env-vars policy,retry_count...
      """,
  )

  userenvvars_group.add_argument(
      '--update-env-vars',
      type=arg_parsers.ArgDict(key_type=str, value_type=str),
      action=arg_parsers.UpdateAction,
      metavar='KEY=VALUE',
      help="""\
        Updates existing or adds new user-defined environment variables based
        on a comma-separated list of key-value pairs.

        Example:
        gcloud workflows deploy ${workflow_name} --update-env-vars policy=regional,retry_count=2
      """,
  )


def ParseExecution(args):
  """Get and validate execution from the args."""
  return args.CONCEPTS.execution.Parse()


def ParseExecutionLabels(args):
  """Get and validate execution labels from the args."""
  messages = apis.GetClientInstance('workflowexecutions', 'v1').MESSAGES_MODULE
  return labels_util.ParseCreateArgs(args, messages.Execution.LabelsValue)


def ParseWorkflow(args):
  """Get and validate workflow from the args."""
  return args.CONCEPTS.workflow.Parse()


def SetSource(args, workflow, updated_fields):
  """Set source for the workflow based on the arguments.

  Also update updated_fields accordingly.
  Currently only local source file is supported.

  Args:
    args: Args passed to the command.
    workflow: The workflow in which to set the source configuration.
    updated_fields: A list to which an appropriate source field will be added.
  """
  if args.source:
    try:
      workflow.sourceContents = files.ReadFileContents(args.source)
    except files.MissingFileError:
      raise exceptions.BadArgumentException(
          '--source', 'specified file does not exist.'
      )
    updated_fields.append('sourceContents')


def SetDescription(args, workflow, updated_fields):
  """Set description for the workflow based on the arguments.

  Also update updated_fields accordingly.

  Args:
    args: Args passed to the command.
    workflow: The workflow in which to set the description.
    updated_fields: A list to which a description field will be added if needed.
  """
  if args.description is not None:
    workflow.description = args.description
    updated_fields.append('description')


def SetServiceAccount(args, workflow, updated_fields):
  """Set service account for the workflow based on the arguments.

  Also update updated_fields accordingly.

  Args:
    args: Args passed to the command.
    workflow: The workflow in which to set the service account.
    updated_fields: A list to which a service_account field will be added if
      needed.
  """
  if args.service_account is not None:
    prefix = ''
    if not args.service_account.startswith('projects/'):
      prefix = 'projects/-/serviceAccounts/'
    workflow.serviceAccount = prefix + args.service_account
    updated_fields.append('serviceAccount')


def SetLabels(labels, workflow, updated_fields):
  """Set labels for the workflow based on the arguments.

  Also update updated_fields accordingly.

  Args:
    labels: Labels parsed as string to be set on the workflow, or None in case
      the field shouldn't be set.
    workflow: The workflow in which to set the labels.
    updated_fields: A list to which a labels field will be added if needed.
  """
  if labels is not None:
    workflow.labels = labels
    updated_fields.append('labels')


def SetUserEnvVars(env_vars, workflow, updated_fields):
  """Sets user-defined environment variables.

  Also updates updated_fields accordingly.

  Args:
    env_vars: Parsed environment variables to be set on the workflow.
    workflow: The workflow in which to set the User Envrionment Variables.
    updated_fields: A list to which the userEnvVars field will be added if
      needed.
  """
  if env_vars is None:
    return
  workflow.userEnvVars = None if env_vars is CLEAR_ENVIRONMENT else env_vars
  updated_fields.append('userEnvVars')


def UpdateUserEnvVars(env_vars, workflow, updated_fields):
  """Updates user-defined environment variables.

  Also updates updated_fields accordingly.

  Args:
    env_vars: Parsed environment variables to be set on the workflow.
    workflow: The workflow in which to set the User Envrionment Variables.
    updated_fields: A list to which the userEnvVars field will be added if
      needed.
  """
  if env_vars is None:
    return
  env_vars_cls = apis.GetClientInstance(
      'workflows',
      'v1',
  ).MESSAGES_MODULE.Workflow.UserEnvVarsValue
  workflow.userEnvVars = env_vars_cls(
      additionalProperties=[
          env_vars_cls.AdditionalProperty(key=key, value=value)
          for key, value in sorted(env_vars.items())
      ]
  )
  updated_fields.append('userEnvVars')



# -*- coding: utf-8 -*- #
# Copyright 2024 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Custom errors for SCC IaC Remediation."""


from googlecloudsdk.api_lib.scc.iac_remediation import const
from googlecloudsdk.core import exceptions


class Error(exceptions.Error):
  """Base error for this module."""


class InvalidFindingNameError(Error):
  """An error respresenting an invalid finding canonical format name error."""

  def __init__(self, bad_finding_name: str):
    if bad_finding_name is None:
      super(Error, self).__init__('Missing finding canonical name.')
    else:
      super(Error, self).__init__(
          f"""Invalid finding canonical name: {bad_finding_name}
          Correct format: projects/*/sources/*/locations/*/findings/*"""
      )


class UnsupportedFindingCategoryError(Error):
  """An error representing an unsupported finding category error."""

  def __init__(self, invalid_category_name: str):
    if invalid_category_name is None:
      super(Error, self).__init__('Missing finding category name.')
    else:
      super(Error, self).__init__(
          f'Finding category not supported: {invalid_category_name}.'
      )


class FindingNotFoundError(Error):
  """An error representing a SCC finding not found error."""

  def __init__(self):
    super(Error, self).__init__(
        'Finding not found for the given name and organization.'
    )


class GitRepoNotFoundError(Error):
  """An error representing a git repo not found error."""

  def __init__(self):
    super(Error, self).__init__(
        'Command is being invoked from a non-git repo'
    )


class InvalidGitConfigError(Error):
  """An error representing missing field in the git config file."""

  def __init__(self, missing_field: str):
    if missing_field is None:
      super(Error, self).__init__('Missing git config field name.')
    else:
      super(Error, self).__init__(
          f'Field missing from the git config file: {missing_field}.'
      )


class EmptyLLMResponseError(Error):
  """An error representing an invalid LLM response error."""

  def __init__(self):
    super(Error, self).__init__(
        'LLM response is empty.'
    )


class InvalidLLMResponseError(Error):
  """An error representing an invalid LLM response error."""

  def __init__(self, error_message: str):
    if error_message is None:
      super(Error, self).__init__('Invalid LLM response.')
    else:
      super(Error, self).__init__(f'Invalid LLM response: {error_message}')


class ExcessiveMembersError(Error):
  """An error representing an excessive members error."""

  def __init__(self, num_members: int):
    super(Error, self).__init__(
        f'Excessive members in the finding: {num_members}, expected atmost'
        f' {const.SUPPORTED_IAM_MEMBER_COUNT_LIMIT}.'
    )



# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Command for getting the status of Compute Engine VPN Gateways."""


from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute import flags as compute_flags
from googlecloudsdk.command_lib.compute.vpn_gateways import flags


class Describe(base.DescribeCommand):
  """Get status of a Compute Engine Highly Available VPN Gateway.

  *{command}* is used to display high availability configuration status for the
  Cloud VPN gateway, the command will show you the high availability
  configuration status for VPN tunnels associated with each peer gateway
  to which the Cloud VPN gateway is connected; the peer gateway could be either
  a Cloud VPN gateway or an external VPN gateway.
  """

  detailed_help = {
      'EXAMPLES':
          """\
          To get status of a VPN gateway, run:

              $ {command} my-gateway --region=us-central1"""
  }

  VPN_GATEWAY_ARG = None

  @staticmethod
  def Args(parser):
    Describe.VPN_GATEWAY_ARG = flags.GetVpnGatewayArgument()
    Describe.VPN_GATEWAY_ARG.AddArgument(parser, operation_type='describe')

  def Run(self, args):
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    client = holder.client

    gateway_ref = Describe.VPN_GATEWAY_ARG.ResolveAsResource(
        args,
        holder.resources,
        scope_lister=compute_flags.GetDefaultScopeLister(client))

    request = client.messages.ComputeVpnGatewaysGetStatusRequest(
        **gateway_ref.AsDict())

    return client.MakeRequests([(client.apitools_client.vpnGateways,
                                 'GetStatus', request)])[0]



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for listing the rules of organization firewall policies."""


from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.compute import firewalls_utils
from googlecloudsdk.api_lib.compute import lister
from googlecloudsdk.api_lib.compute.firewall_policies import client
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute.firewall_policies import firewall_policies_utils
from googlecloudsdk.command_lib.compute.firewall_policies import flags
from googlecloudsdk.core import log
import six

LIST_NOTICE = """\
To show all fields of the firewall, please show in JSON format: --format=json
To show more fields in table format, please see the examples in --help.
"""

DEFAULT_LIST_FORMAT = """\
  table(
    priority,
    direction,
    action,
    match.srcIpRanges.list():label=SRC_RANGES,
    match.destIpRanges.list():label=DEST_RANGES,
    match.layer4Configs.map().org_firewall_rule().list():label=PORT_RANGES
  )"""


@base.DefaultUniverseOnly


class ListRules(base.DescribeCommand, base.ListCommand):
  """List the rules of a Compute Engine organization firewall policy.

  *{command}* is used to list the rules of an organization firewall policy.
  """

  FIREWALL_POLICY_ARG = None

  @classmethod
  def Args(cls, parser):
    cls.FIREWALL_POLICY_ARG = flags.FirewallPolicyArgument(
        required=True, operation='list rules for')
    cls.FIREWALL_POLICY_ARG.AddArgument(parser, operation_type='get')
    parser.add_argument(
        '--organization',
        help=('Organization which the organization firewall policy belongs to. '
              'Must be set if FIREWALL_POLICY is short name.'))
    parser.display_info.AddFormat(DEFAULT_LIST_FORMAT)
    lister.AddBaseListerArgs(parser)

  def Run(self, args):
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    ref = self.FIREWALL_POLICY_ARG.ResolveAsResource(
        args, holder.resources, with_project=False)
    org_firewall_policy = client.OrgFirewallPolicy(
        ref=ref,
        compute_client=holder.client,
        resources=holder.resources,
        version=six.text_type(self.ReleaseTrack()).lower())
    fp_id = firewall_policies_utils.GetFirewallPolicyId(
        org_firewall_policy, ref.Name(), organization=args.organization)
    response = org_firewall_policy.Describe(
        fp_id=fp_id, only_generate_request=False)
    if not response:
      return None
    return firewalls_utils.SortFirewallPolicyRules(holder.client,
                                                   response[0].rules)

  def Epilog(self, resources_were_displayed):
    del resources_were_displayed
    log.status.Print('\n' + LIST_NOTICE)


ListRules.detailed_help = {
    'EXAMPLES':
        """\
    To list the rules of an organization firewall policy with ID
    ``123456789", run:

      $ {command} 123456789

    To list all the fields of the rules of an organization firewall policy with
    ID ``123456789", run:

      $ {command} 123456789 --format="table(
        priority,
        action,
        direction,
        match.srcIpRanges.list():label=SRC_RANGES,
        match.destIpRanges.list():label=DEST_RANGES,
        match.layer4Configs.map().org_firewall_rule().list():label=PORT_RANGES,
        targetServiceAccounts.list():label=TARGET_SVC_ACCT,
        targetResources:label=TARGET_RESOURCES,
        ruleTupleCount,
        enableLogging,
        description)"
        """,
    'IAM PERMISSIONS': """\
    To list rules of a firewall policy, the user must have the following
    permission: *`compute.firewallPolicies.get`.

    To find predefined roles that contain those permissions, see the [Compute
    Engine IAM roles](https://cloud.google.com/compute/docs/access/iam).
    """
}



# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Implements command to describe a specified guest policy."""


from googlecloudsdk.api_lib.compute.os_config import utils as osconfig_api_utils
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute.os_config import utils as osconfig_command_utils
from googlecloudsdk.core import properties


@base.ReleaseTracks(base.ReleaseTrack.BETA)
class Describe(base.DescribeCommand):
  """Describe the specified guest policy.

  ## EXAMPLES

    To describe the guest policy `policy1` in the current project, run:

          $ {command} policy1

  """

  @staticmethod
  def Args(parser):
    """See base class."""
    parser.add_argument(
        'POLICY_ID', type=str, help='Name of the guest policy to describe.')

  def Run(self, args):
    """See base class."""
    release_track = self.ReleaseTrack()
    client = osconfig_api_utils.GetClientInstance(release_track)
    messages = osconfig_api_utils.GetClientMessages(release_track)

    project = properties.VALUES.core.project.GetOrFail()
    request = messages.OsconfigProjectsGuestPoliciesGetRequest(
        name=osconfig_command_utils.GetGuestPolicyUriPath(
            'projects', project, args.POLICY_ID))
    service = client.projects_guestPolicies

    return service.Get(request)



# -*- coding: utf-8 -*- #
# Copyright 2024 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to generate a new Audit Scope."""


from apitools.base.py import exceptions as apitools_exceptions
from googlecloudsdk.api_lib.audit_manager import audit_scopes
from googlecloudsdk.api_lib.audit_manager import constants
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.audit_manager import exception_utils
from googlecloudsdk.command_lib.audit_manager import flags
from googlecloudsdk.core import exceptions as core_exceptions
from googlecloudsdk.core import properties
from googlecloudsdk.core.util import files


_DETAILED_HELP = {
    'DESCRIPTION': 'Generate a new Audit Scope.',
    'EXAMPLES': """ \
        To generate an Audit Scope in the `us-central1` region,
        for a project with ID `123` for compliance framework `fedramp_moderate` in `odf` format, run:

          $ {command} --project="123" --location="us-central1" --compliance-framework="fedramp_moderate" --report-format="odf" --output-directory="scopes/currentyear" --output-file-name="auditreport"
        """,
}

_SCOPE_REPORT_CONTENTS = 'scopeReportContents'
_FILE_EXTENSION = '.ods'


@base.DefaultUniverseOnly
@base.ReleaseTracks(base.ReleaseTrack.GA)
class Generate(base.CreateCommand):
  """Generate Audit Scope."""

  detailed_help = _DETAILED_HELP
  api_version = constants.ApiVersion.V1

  @staticmethod
  def Args(parser):
    flags.AddProjectOrFolderFlags(parser, 'for which to generate audit scope')
    flags.AddLocationFlag(parser, 'the scope should be generated')
    flags.AddComplianceFrameworkFlag(parser)
    flags.AddScopeReportFormatFlag(parser)
    flags.AddOutputDirectoryFormatFlag(parser)
    flags.AddOutputFileNameFormatFlag(parser)

  def Run(self, args):
    """Run the generate command."""
    is_parent_folder = args.folder is not None

    scope = (
        'folders/{folder}'.format(folder=args.folder)
        if is_parent_folder
        else 'projects/{project}'.format(project=args.project)
    )
    scope += '/locations/{location}'.format(location=args.location)

    client = audit_scopes.AuditScopesClient(api_version=self.api_version)
    try:
      response = client.Generate(
          scope,
          args.compliance_framework,
          report_format=args.report_format,
          is_parent_folder=is_parent_folder,
      )
      self.SaveReport(
          response,
          args.output_directory,
          args.output_file_name,
      )
      return response

    except apitools_exceptions.HttpError as error:
      exc = exception_utils.AuditManagerError(error)

      if exc.has_error_info(exception_utils.ERROR_REASON_PERMISSION_DENIED):
        role = 'roles/auditmanager.auditor'
        user = properties.VALUES.core.account.Get()
        exc.suggested_command_purpose = 'grant permission'
        command_prefix = (
            'gcloud resource-manager folders add-iam-policy-binding'
            if is_parent_folder
            else 'gcloud projects add-iam-policy-binding'
        )
        exc.suggested_command = (
            f'{command_prefix}'
            f' {args.folder if is_parent_folder else args.project}'
            f' --member=user:{user} --role {role}'
        )

      core_exceptions.reraise(exc)

  # TODO: b/324031367 - Add type annotations to all the methods.
  def SaveReport(self, response, output_directory, output_file_name):
    """Save the generated scope."""
    is_empty_directory_path = output_directory == ''
    directory_path = '' if is_empty_directory_path else output_directory + '/'
    file_path = directory_path + output_file_name + _FILE_EXTENSION
    content_bytes = response.scopeReportContents
    files.WriteBinaryFileContents(file_path, content_bytes, overwrite=False)


@base.DefaultUniverseOnly
@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class GenerateAlpha(Generate):
  """Generate Audit Scope."""

  api_version = constants.ApiVersion.V1_ALPHA



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Implements the command to list vulnerabilities from Artifact Registry."""


import re
from googlecloudsdk.api_lib.artifacts import exceptions as ar_exceptions
from googlecloudsdk.api_lib.artifacts.vulnerabilities import GetLatestScan
from googlecloudsdk.api_lib.artifacts.vulnerabilities import GetVulnerabilities
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.artifacts import docker_util
from googlecloudsdk.command_lib.artifacts import endpoint_util
from googlecloudsdk.command_lib.artifacts import flags
from googlecloudsdk.command_lib.artifacts import format_util


DEFAULT_LIST_FORMAT = """\
     table[box, title="%TITLE%"](
      occurrence.vulnerability.shortDescription:label=CVE,
      occurrence.vulnerability.effectiveSeverity:label=EFFECTIVE_SEVERITY,
      occurrence.vulnerability.cvssScore:label=CVSS:sort=-1:reverse,
      occurrence.vulnerability.packageIssue.fixAvailable:label=FIX_AVAILABLE,
      occurrence.vulnerability.vexAssessment.state:label=VEX_STATUS,
      occurrence.vulnerability.packageIssue.affectedPackage:sort=3:label=PACKAGE,
      occurrence.vulnerability.packageIssue.packageType:label=PACKAGE_TYPE,
      vexScope,
      {}
    )
    """.format(format_util.CONTAINER_ANALYSIS_METADATA_FORMAT)


@base.ReleaseTracks(base.ReleaseTrack.GA)
@base.DefaultUniverseOnly
class List(base.ListCommand):
  """Command for listing vulnerabilities. To see all fields, use --format=json.
  """

  detailed_help = {
      'DESCRIPTION': '{description}',
      'EXAMPLES': """\
        To list vulnerabilities for an artifact, run:

          $ {command} us-east1-docker.pkg.dev/project123/repository123/someimage@sha256:49765698074d6d7baa82f
      """,
  }

  @staticmethod
  def Args(parser):
    flags.GetListURIArg().AddToParser(parser)
    flags.GetOptionalAALocationFlag().AddToParser(parser)
    flags.GetVulnerabilitiesOccurrenceFilterFlag().AddToParser(parser)
    parser.display_info.AddFlatten(['occurrence.vulnerability.packageIssue'])
    return

  def Run(self, args):
    location = args.location
    occurrence_filter = args.occurrence_filter
    resource, project = self.replaceTags(args.URI)
    with endpoint_util.WithRegion(location):
      if location is not None:
        project = '{}/locations/{}'.format(project, location)
      latest_scan = GetLatestScan(project, resource)
      occurrences = GetVulnerabilities(project, resource, occurrence_filter)
    self.setTitle(args, latest_scan)
    occurrences = list(occurrences)
    results = []
    if len(occurrences) < 1:
      return {}
    for occ in occurrences:
      vex_scope = ''
      if (
          occ.vulnerability
          and occ.vulnerability.vexAssessment
          and occ.vulnerability.vexAssessment.noteName
      ):
        tokens = occ.vulnerability.vexAssessment.noteName.split('/')
        if tokens[-1].startswith('image-'):
          vex_scope = 'IMAGE'
        else:
          vex_scope = 'DIGEST'
      results.append(VulnerabilityEntry(occ, vex_scope))
    return results

  def replaceTags(self, original_uri):
    updated_uri = original_uri
    if not updated_uri.startswith('https://'):
      updated_uri = 'https://{}'.format(updated_uri)
    found = re.findall(docker_util.DOCKER_URI_REGEX, updated_uri)
    if found:
      resource_uri_str = found[0][0]
      is_gcr = 'gcr.io' in found[0][1]
      if is_gcr:
        resource_uri_str, _, _ = docker_util.ConvertGCRImageString(
            resource_uri_str,
        )
      image, version = docker_util.DockerUrlToVersion(resource_uri_str)
      if is_gcr:
        version = docker_util.GcrDockerVersion(
            image.docker_repo.project,
            version.GetDockerString().replace(
                image.docker_repo.GetDockerString(),
                '{}/{}'.format(
                    image.docker_repo.repo,  # AR repo name is the gcr_host
                    image.docker_repo.project,
                ),
            ),
        )
      project = image.project
      docker_html_str_digest = 'https://{}'.format(version.GetDockerString())
      updated_uri = re.sub(
          docker_util.DOCKER_URI_REGEX,
          docker_html_str_digest,
          updated_uri,
          1,
      )
      return updated_uri, project
    raise ar_exceptions.InvalidInputValueError(
        'Received invalid URI {}'.format(original_uri)
    )

  def setTitle(self, args, latest_scan):
    title = ''
    if (
        not latest_scan
        or latest_scan.discovery is None
        or latest_scan.discovery.lastScanTime is None
    ):
      title = 'Scan status unknown'
    else:
      last_scan_time = latest_scan.discovery.lastScanTime[:-11]
      title = 'Latest scan was at {}'.format(last_scan_time)
    list_format = DEFAULT_LIST_FORMAT.replace('%TITLE%', title)
    args.GetDisplayInfo().AddFormat(list_format)


class VulnerabilityEntry(object):
  """Holder for an entry of vulnerability list results.

  Properties:
    occurrence: Vulnerability occurrence.
    vex_scope: Scope of the VEX statement.
  """

  def __init__(self, occurrence, vex_scope):
    self._occurrence = occurrence
    self._vex_scope = vex_scope

  @property
  def occurrence(self):
    return self._occurrence

  @property
  def vex_scope(self):
    return self._vex_scope



# -*- coding: utf-8 -*- #
# Copyright 2024 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Client for interaction with Governance Rules API CRUD DATAPLEX."""


import os

from googlecloudsdk.api_lib.dataplex import util as dataplex_api
from googlecloudsdk.api_lib.util import messages as messages_util
from googlecloudsdk.calliope import exceptions
from googlecloudsdk.command_lib.iam import iam_util
from googlecloudsdk.core import yaml
from googlecloudsdk.core.util import files
import six


def GenerateGovernanceRuleForCreateRequest(args):
  """Create Governance Rule Requests."""
  module = dataplex_api.GetMessageModule()
  request = module.GoogleCloudDataplexV1GovernanceRule(
      description=args.description,
      displayName=args.display_name,
      labels=dataplex_api.CreateLabels(
          module.GoogleCloudDataplexV1GovernanceRule, args
      ),
      query=GenerateGovernanceRuleQuery(args),
      specs=GenerateGovernanceRuleSpecs(args),
      fields=GenerateGovernanceRuleFields(args),
  )

  if args.IsSpecified('rule_metadata_file'):
    rule_metadata_file = ReadRuleMetadataFile(args)
    if rule_metadata_file is None:
      raise ValueError(
          'Rule metadata file is empty for Governance Rules create request.'
      )
    if rule_metadata_file.get('query') is None:
      raise ValueError(
          'Query should be provided for Governance Rules create request.'
      )
    if GenerateGovernanceRuleSpecs(
        args
    ) is None and not GenerateGovernanceRuleFields(args):
      raise ValueError(
          'Either specs or field should be provided for Governance Rules create'
          ' request.'
      )
  else:
    raise ValueError(
        'Rule metadata file is not specified for Governance Rules create'
        ' request.'
    )

  return request


def GenerateGovernanceRuleForUpdateRequest(args):
  """Update Governance Rule Requests."""
  module = dataplex_api.GetMessageModule()
  request = module.GoogleCloudDataplexV1GovernanceRule(
      description=args.description,
      displayName=args.display_name,
      etag=args.etag,
      labels=dataplex_api.CreateLabels(
          module.GoogleCloudDataplexV1GovernanceRule, args
      ),
  )
  if (
      args.IsSpecified('rule_metadata_file')
      and args.rule_metadata_file is not None
  ):
    query = GenerateGovernanceRuleQuery(args)
    specs = GenerateGovernanceRuleSpecs(args)
    fields = GenerateGovernanceRuleFields(args)
    if query is not None:
      request.query = query
    if specs is not None:
      request.specs = specs
    if fields:
      request.fields = fields
  return request


def ReadRuleMetadataFile(args):
  """Read Rule Metadata File."""
  if not os.path.exists(args.rule_metadata_file):
    raise exceptions.BadFileException(
        'No such file [{0}]'.format(args.rule_metadata_file)
    )
  if os.path.isdir(args.rule_metadata_file):
    raise exceptions.BadFileException(
        '[{0}] is a directory'.format(args.rule_metadata_file)
    )
  try:
    with files.FileReader(args.rule_metadata_file) as import_file:
      return yaml.load(import_file)
  except Exception as exp:
    exp_msg = getattr(exp, 'message', six.text_type(exp))
    msg = (
        'Unable to read Rule Metadata config from specified file '
        '[{0}] because [{1}]'.format(args.rule_metadata_file, exp_msg)
    )
    raise exceptions.BadFileException(msg)


def GenerateGovernanceRuleQuery(args):
  """Generate Governance Rule Query From Rule Metadata File."""
  module = dataplex_api.GetMessageModule()
  governance_rule_query = module.GoogleCloudDataplexV1GovernanceRuleQuery()
  rule_metadata_file = ReadRuleMetadataFile(args)
  if (
      rule_metadata_file is not None
      and rule_metadata_file.get('query') is not None
  ):
    governance_rule_query = messages_util.DictToMessageWithErrorCheck(
        dataplex_api.SnakeToCamelDict(rule_metadata_file.get('query')),
        module.GoogleCloudDataplexV1GovernanceRuleQuery,
        True,
    )
  return governance_rule_query


def GenerateGovernanceRuleSpecs(args):
  """Generate Governance Rule Specs From Rule Metadata File."""
  module = dataplex_api.GetMessageModule()
  governance_rule_specs = None
  rule_metadata_file = ReadRuleMetadataFile(args)
  if (
      rule_metadata_file is not None
      and rule_metadata_file.get('specs') is not None
  ):
    governance_rule_specs = messages_util.DictToMessageWithErrorCheck(
        dataplex_api.SnakeToCamelDict(rule_metadata_file.get('specs')),
        module.GoogleCloudDataplexV1GovernanceRuleSpecs,
    )
  return governance_rule_specs


def GenerateGovernanceRuleFields(args):
  """Generate Governance Rule Fields From Rule Metadata File."""
  module = dataplex_api.GetMessageModule()
  governance_rule_fields = []
  rule_metadata_file = ReadRuleMetadataFile(args)
  if (
      rule_metadata_file is not None
      and rule_metadata_file.get('fields') is not None
  ):
    fields = rule_metadata_file.get('fields')
    for field in fields:
      governance_rule_fields.append(
          messages_util.DictToMessageWithErrorCheck(
              dataplex_api.SnakeToCamelDict(field),
              module.GoogleCloudDataplexV1GovernanceRuleField,
          )
      )
  return governance_rule_fields


def GenerateUpdateMask(args):
  """Create Update Mask for Governance Rule."""
  update_mask = []
  if args.IsSpecified('description'):
    update_mask.append('description')
  if args.IsSpecified('display_name'):
    update_mask.append('displayName')
  if args.IsSpecified('labels'):
    update_mask.append('labels')

  if args.IsSpecified('rule_metadata_file'):
    if args.rule_metadata_file is not None:
      if GenerateGovernanceRuleQuery(args) is not None:
        update_mask.append('query')
      if GenerateGovernanceRuleSpecs(args) is not None:
        update_mask.append('specs')
      if GenerateGovernanceRuleFields(args):
        update_mask.append('fields')
  return update_mask


def WaitForOperation(operation):
  """Waits for the given google.longrunning.Operation to complete."""
  return dataplex_api.WaitForOperation(
      operation,
      dataplex_api.GetClientInstance().projects_locations_governanceRules,
  )


def SetIamPolicy(governance_rule_ref, policy):
  """Set Iam Policy request."""
  google_iam_v1_set_iam_policy_request = (
      dataplex_api.GetMessageModule().GoogleIamV1SetIamPolicyRequest(
          policy=policy
      )
  )
  set_iam_policy_req = dataplex_api.GetMessageModule().DataplexProjectsLocationsGovernanceRulesSetIamPolicyRequest(
      resource=governance_rule_ref.RelativeName(),
      googleIamV1SetIamPolicyRequest=google_iam_v1_set_iam_policy_request,
  )

  return dataplex_api.GetClientInstance().projects_locations_governanceRules.SetIamPolicy(
      set_iam_policy_req
  )


def GetIamPolicy(governance_rule_ref):
  """Get Iam Policy request."""
  get_iam_policy_req = dataplex_api.GetMessageModule().DataplexProjectsLocationsGovernanceRulesGetIamPolicyRequest(
      resource=governance_rule_ref.RelativeName()
  )
  return dataplex_api.GetClientInstance().projects_locations_governanceRules.GetIamPolicy(
      get_iam_policy_req
  )


def AddIamPolicyBinding(governance_rule_ref, member, role):
  """Add IAM policy binding request."""
  policy = GetIamPolicy(governance_rule_ref)
  iam_util.AddBindingToIamPolicy(
      dataplex_api.GetMessageModule().GoogleIamV1Binding, policy, member, role
  )
  return SetIamPolicy(governance_rule_ref, policy)


def RemoveIamPolicyBinding(governance_rule_ref, member, role):
  """Remove IAM policy binding request."""
  policy = GetIamPolicy(governance_rule_ref)
  iam_util.RemoveBindingFromIamPolicy(policy, member, role)
  return SetIamPolicy(governance_rule_ref, policy)


def SetIamPolicyFromFile(governance_rule_ref, policy_file):
  """Set IAM policy binding request from file."""
  policy = iam_util.ParsePolicyFile(
      policy_file, dataplex_api.GetMessageModule().GoogleIamV1Policy
  )
  return SetIamPolicy(governance_rule_ref, policy)



# -*- coding: utf-8 -*- #
# Copyright 2017 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Surface for listing all SSL certificates for an App Engine app."""


from googlecloudsdk.api_lib.app.api import appengine_ssl_api_client as api_client
from googlecloudsdk.calliope import base


@base.ReleaseTracks(base.ReleaseTrack.GA)
class List(base.ListCommand):
  """Lists the SSL certificates."""

  detailed_help = {
      'DESCRIPTION':
          '{description}',
      'EXAMPLES':
          """\
          To list all App Engine SSL certificates, run:

              $ {command}

          This will return certificates mapped to domain-mappings for the
          current app as well as all certificates that apply to domains which
          the current user owns.

          To view your owned domains, run `gcloud domains list-user-verified`.
          """,
  }

  def Run(self, args):
    return api_client.GetApiClientForTrack(
        self.ReleaseTrack()).ListSslCertificates()

  @staticmethod
  def Args(parser):
    parser.display_info.AddFormat("""
            table(
              id:sort=1,
              display_name,
              domain_names.list()
            )
        """)


@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA)
class ListBeta(List):

  @staticmethod
  def Args(parser):
    parser.display_info.AddFormat("""
            table(
              id:sort=1,
              display_name,
              domain_names.list(),
              managed_certificate.status:label=MANAGED_CERTIFICATE_STATUS
            )
        """)



# -*- coding: utf-8 -*- #
# Copyright 2013 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""The command to restore a backup of a Google Cloud CLI installation."""


from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.components import util


@base.Hidden
@base.Deprecate(
    is_removed=False,
    warning=(
        'Starting with release 473.0.0, the Google Cloud CLI updates in place'
        ' instead of making a backup copy of the installation directory when'
        ' running `gcloud components update`, `install`, and `remove` commands'
        ' (this reduces the time taken by those operations). Consequently, this'
        ' command has no backup to restore in those cases. Instead, to restore'
        ' your installation to a previous version, run'
        ' `gcloud components update --version=<previous_version>`, or install'
        ' the previous version directly from'
        ' https://cloud.google.com/sdk/docs/install.'
    ),
)
@base.UniverseCompatible
class Restore(base.SilentCommand):
  """Restore the Google Cloud CLI installation to its state before a reinstall.

  This is an undo operation, which restores the Google Cloud CLI installation on
  the local workstation to the state it was in just before the most recent
  `{parent_command} reinstall` command. A `restore` command does not undo a
  previous `restore` command.

  ## EXAMPLES
  To restore the Google Cloud CLI installation to its state before reinstalling,
  run:

    $ {command}
  """

  @staticmethod
  def Args(_):
    pass

  def Run(self, args):
    """Runs the restore command."""
    update_manager = util.GetUpdateManager(args)
    update_manager.Restore()



# -*- coding: utf-8 -*- #
# Copyright 2026 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Translation rule for app resources (volumes)."""

from collections.abc import Mapping, Sequence
from typing import Any
from googlecloudsdk.command_lib.app.gae_to_cr_migration_util.common import util


def translate_volumes(input_data: Mapping[str, Any]) -> Sequence[str]:
  """Translates resources.volumes to Cloud Run --add-volume flags.

  Args:
    input_data: Flattened dictionary of the input data from app.yaml.

  Returns:
    List of output flags.
  """
  if not util.is_flex_env(input_data):
    return []

  volumes = input_data.get('resources.volumes')
  if not volumes:
    return []

  flags = []
  for volume in volumes:
    name = volume.get('name')
    if not name:
      continue
    volume_type = volume.get('volume_type')
    size_gb = volume.get('size_gb')

    if volume_type == 'tmpfs':
      # Cloud Run in-memory volumes use size-limit in Mi or Gi.
      # Flex uses size_gb in float. 1GB = 1024Mi.
      size_mi = int(float(size_gb) * 1024) if size_gb else 512
      flags.append(
          f'--add-volume=name={name},type=in-memory,size-limit={size_mi}Mi'
      )
      flags.append(f'--add-volume-mount=volume={name},mount-path=/mnt/{name}')

  return flags



# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""`gcloud access-context-manager perimeters dry-run delete` command."""


from googlecloudsdk.api_lib.accesscontextmanager import zones as zones_api
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.accesscontextmanager import perimeters
from googlecloudsdk.command_lib.accesscontextmanager import policies


@base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.GA)
class DeletePerimeterDryRun(base.UpdateCommand):
  """Marks the Service Perimeter for deletion in the dry-run mode."""
  _API_VERSION = 'v1'

  @staticmethod
  def Args(parser):
    perimeters.AddResourceArg(parser, 'to delete')
    parser.add_argument(
        '--async',
        action='store_true',
        help="""Return immediately, without waiting for the operation in
                progress to complete.""")

  def Run(self, args):
    client = zones_api.Client(version=self._API_VERSION)
    perimeter_ref = args.CONCEPTS.perimeter.Parse()
    policies.ValidateAccessPolicyArg(perimeter_ref, args)
    return client.UnsetSpec(perimeter_ref, use_explicit_dry_run_spec=True)


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class DeletePerimeterDryRunAlpha(DeletePerimeterDryRun):
  """Marks the Service Perimeter for deletion in the dry-run mode."""
  _API_VERSION = 'v1alpha'


detailed_help = {
    'brief':
        'Mark the Service Perimeter as deleted in the dry-run mode.',
    'DESCRIPTION':
        ('When this command completed successfully, the affected Service '
         'Perimeter will be considered to have been deleted in the dry-run '
         'mode, but the enforcement mode configuration will be left untouched.'
        ),
    'EXAMPLES':
        ('To mark the Service Perimeter as deleted in the dry-run mode:\n\n'
         '  $ {command} my-perimeter')
}

DeletePerimeterDryRunAlpha.detailed_help = detailed_help
DeletePerimeterDryRun.detailed_help = detailed_help



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Base ResourceBuilder for Cloud Run Integrations."""


from typing import Dict, List, Optional

from apitools.base.py import encoding
from googlecloudsdk.api_lib.run.integrations import types_utils
from googlecloudsdk.command_lib.run.integrations.typekits import base
from googlecloudsdk.command_lib.runapps import exceptions
from googlecloudsdk.generated_clients.apis.runapps.v1alpha1 import runapps_v1alpha1_messages

DOMAIN_TYPE = 'domain'


class CustomDomainsTypeKit(base.TypeKit):
  """The domain routing integration typekit."""

  def GetDeployMessage(self, create=False):
    message = 'This might take up to 5 minutes.'

    if create:
      message += ' Manual DNS configuration will be required after completion.'
    return message

  def UpdateResourceConfig(
      self,
      parameters: Dict[str, str],
      resource: runapps_v1alpha1_messages.Resource,
  ) -> List[str]:
    """Updates the resource config according to the parameters.

    Args:
      parameters: parameters from the command
      resource: the resource object of the integration

    Returns:
      list of service names referred in parameters.
    """
    services = []
    if 'set-mapping' in parameters:
      url, service = self._ParseMappingNotation(parameters.get('set-mapping'))
      services.append(service)
      svc_id = runapps_v1alpha1_messages.ResourceID(
          name=service, type='service'
      )
      domain, path = self._ParseDomainPath(url)
      domain_config = self._FindDomainConfig(resource.subresources, domain)
      if domain_config is None:
        domain_res_name = self._DomainResourceName(domain)
        domain_config = runapps_v1alpha1_messages.Resource(
            id=runapps_v1alpha1_messages.ResourceID(
                type=DOMAIN_TYPE, name=domain_res_name
            ),
            config=encoding.DictToMessage(
                {'domain': domain},
                runapps_v1alpha1_messages.Resource.ConfigValue,
            ),
        )
        resource.subresources.append(domain_config)
      if path != '/*' and not domain_config.bindings:
        raise exceptions.ArgumentError('New domain must map to root path')
      # If path already set to other service, remove it.
      self._RemovePath(domain_config, path)

      bindings = base.FindBindings(
          domain_config, types_utils.SERVICE_TYPE, service
      )
      if bindings:
        binding = bindings[0]
        cfg = encoding.MessageToDict(binding.config)
        cfg.setdefault('paths', []).append(path)
        binding.config = encoding.DictToMessage(
            cfg, runapps_v1alpha1_messages.Binding.ConfigValue
        )
      else:
        domain_config.bindings.append(
            runapps_v1alpha1_messages.Binding(
                targetRef=runapps_v1alpha1_messages.ResourceRef(id=svc_id),
                config=encoding.DictToMessage(
                    {'paths': [path]},
                    runapps_v1alpha1_messages.Binding.ConfigValue,
                ),
            )
        )

    elif 'remove-mapping' in parameters:
      url = parameters.get('remove-mapping')
      if ':' in url:
        raise exceptions.ArgumentError(
            'Service notion not allowed in remove-mapping'
        )
      domain, path = self._ParseDomainPath(url)
      domain_config = self._FindDomainConfig(resource.subresources, domain)
      if domain_config is None:
        raise exceptions.ArgumentError(
            'Domain "{}" does not exist'.format(domain)
        )
      if path == '/*':
        # Removing root route
        if len(domain_config.bindings) > 1:
          # If the root route is not the only route, it can't be removed.
          raise exceptions.ArgumentError(
              (
                  'Can not remove root route of domain "{}" '
                  + 'because there are other routes configured.'
              ).format(domain)
          )
        else:
          # If it's the only route, delete the whole domain.
          resource.subresources.remove(domain_config)
      else:
        # Removing non-root route
        self._RemovePath(domain_config, path)
    elif 'remove-domain' in parameters:
      domain = parameters['remove-domain'].lower()
      domain_config = self._FindDomainConfig(resource.subresources, domain)
      if domain_config is None:
        raise exceptions.ArgumentError(
            'Domain "{}" does not exist'.format(domain)
        )
      resource.subresources.remove(domain_config)

    if not resource.subresources:
      raise exceptions.ArgumentError(
          (
              'Can not remove the last domain. '
              + 'Use "gcloud run integrations delete custom-domains" instead.'
          )
      )

    return services

  def BindServiceToIntegration(
      self,
      integration: runapps_v1alpha1_messages.Resource,
      workload: runapps_v1alpha1_messages.Resource,
      parameters: Optional[Dict[str, str]] = None,
  ):
    """Bind a workload to an integration.

    Args:
      integration: the resource of the inetgration.
      workload: the resource the workload.
      parameters: the binding config from parameter.

    Raises:
      exceptions.ArgumentError: always raise this exception because binding
      service is not supported in DomainRouting integration.
    """
    raise exceptions.ArgumentError(
        '--add-service is not supported in custom-domains integration'
    )

  def UnbindServiceFromIntegration(
      self,
      integration: runapps_v1alpha1_messages.Resource,
      workload: runapps_v1alpha1_messages.Resource,
  ):
    """Unbind a workload from an integration.

    Args:
      integration: the resource of the inetgration.
      workload: the resource the workload.

    Raises:
      exceptions.ArgumentError: always raise this exception because unbinding
      service is not supported in DomainRouting integration.
    """
    raise exceptions.ArgumentError(
        '--remove-service is not supported in custom-domains integration'
    )

  def NewIntegrationName(
      self, appconfig: runapps_v1alpha1_messages.Config
  ) -> str:
    """Returns a name for a new integration.

    Args:
      appconfig: the application config

    Returns:
      str, a new name for the integration.
    """
    return self.singleton_name

  def _FindDomainConfig(
      self, subresources: List[runapps_v1alpha1_messages.Resource], domain
  ) -> Optional[runapps_v1alpha1_messages.Resource]:
    for res in subresources:
      if res.id.type == DOMAIN_TYPE:
        cfg = encoding.MessageToDict(res.config)
        if cfg.get('domain') == domain:
          return res
    return None

  def _RemovePath(
      self, domain_res: runapps_v1alpha1_messages.Resource, path: str
  ):
    for route in domain_res.bindings:
      cfg = encoding.MessageToDict(route.config)
      paths = cfg.get('paths')
      for route_path in paths:
        if route_path == path:
          paths.remove(route_path)
          break
      if paths:
        route.config = encoding.DictToMessage(
            cfg, runapps_v1alpha1_messages.Binding.ConfigValue
        )
      else:
        domain_res.bindings.remove(route)

  def _ParseMappingNotation(self, mapping):
    mapping_parts = mapping.split(':')
    if len(mapping_parts) != 2:
      raise exceptions.ArgumentError(
          'Mapping "{}" is not valid. Missing service notation.'.format(mapping)
      )
    url = mapping_parts[0]
    service = mapping_parts[1]
    return url, service

  def _ParseDomainPath(self, url):
    url_parts = url.split('/', 1)
    domain = url_parts[0]
    path = '/*'
    if len(url_parts) == 2:
      path = '/' + url_parts[1]
    return domain.lower(), path

  def _DomainResourceName(self, domain: str) -> str:
    return '-'.join(domain.split('.')).lower()



# -*- coding: utf-8 -*- #
# Copyright 2024 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Helper utilities for fleet packages commands."""

import glob
import itertools
import os
import pathlib

from googlecloudsdk.core import yaml

_DEFAULT_API_VERSION = 'v1alpha'
_RESOURCE_BUNDLE_PROJECT_SEGMENT = 1
_RESOURCE_BUNDLE_LOCATION_SEGMENT = 3

ROLLOUTS_DESCRIBE_ROLLING_TRUNCATED_MESSAGES_FORMAT = """table(info.rolloutStrategyInfo.rollingStrategyInfo.clusters.membership.basename():label=CLUSTER,
                    info.rolloutStrategyInfo.rollingStrategyInfo.clusters.current.version:label=CURRENT_VERSION,
                    info.rolloutStrategyInfo.rollingStrategyInfo.clusters.current.syncState:label=SYNC_STATE,
                    info.rolloutStrategyInfo.rollingStrategyInfo.clusters.desired.version:label=DESIRED_VERSION,
                    info.rolloutStrategyInfo.rollingStrategyInfo.clusters.startTime:label=START_TIME,
                    info.rolloutStrategyInfo.rollingStrategyInfo.clusters.endTime:label=END_TIME,
                    info.rolloutStrategyInfo.rollingStrategyInfo.clusters.state:label=STATE,
                    trim_message():label=MESSAGE)"""

ROLLOUTS_DESCRIBE_ALLATONCE_TRUNCATED_MESSAGES_FORMAT = """table(info.rolloutStrategyInfo.allAtOnceStrategyInfo.clusters.membership.basename():label=CLUSTER,
                    info.rolloutStrategyInfo.allAtOnceStrategyInfo.clusters.current.version:label=CURRENT_VERSION,
                    info.rolloutStrategyInfo.allAtOnceStrategyInfo.clusters.current.syncState:label=SYNC_STATE,
                    info.rolloutStrategyInfo.allAtOnceStrategyInfo.clusters.desired.version:label=DESIRED_VERSION,
                    info.rolloutStrategyInfo.allAtOnceStrategyInfo.clusters.startTime:label=START_TIME,
                    info.rolloutStrategyInfo.allAtOnceStrategyInfo.clusters.endTime:label=END_TIME,
                    info.rolloutStrategyInfo.allAtOnceStrategyInfo.clusters.state:label=STATE,
                    trim_message():label=MESSAGE)"""

ROLLOUTS_DESCRIBE_ROLLING_FULL_MESSAGES_FORMAT = """table(info.rolloutStrategyInfo.rollingStrategyInfo.clusters.membership.basename():label=CLUSTER,
                    info.rolloutStrategyInfo.rollingStrategyInfo.clusters.current.version:label=CURRENT_VERSION,
                    info.rolloutStrategyInfo.rollingStrategyInfo.clusters.current.syncState:label=SYNC_STATE,
                    info.rolloutStrategyInfo.rollingStrategyInfo.clusters.desired.version:label=DESIRED_VERSION,
                    info.rolloutStrategyInfo.rollingStrategyInfo.clusters.startTime:label=START_TIME,
                    info.rolloutStrategyInfo.rollingStrategyInfo.clusters.endTime:label=END_TIME,
                    info.rolloutStrategyInfo.rollingStrategyInfo.clusters.state:label=STATE,
                    all_messages():label=MESSAGES)"""

ROLLOUTS_DESCRIBE_ALLATONCE_FULL_MESSAGES_FORMAT = """table(info.rolloutStrategyInfo.allAtOnceStrategyInfo.clusters.membership.basename():label=CLUSTER,
                    info.rolloutStrategyInfo.allAtOnceStrategyInfo.clusters.current.version:label=CURRENT_VERSION,
                    info.rolloutStrategyInfo.allAtOnceStrategyInfo.clusters.current.syncState:label=SYNC_STATE,
                    info.rolloutStrategyInfo.allAtOnceStrategyInfo.clusters.desired.version:label=DESIRED_VERSION,
                    info.rolloutStrategyInfo.allAtOnceStrategyInfo.clusters.startTime:label=START_TIME,
                    info.rolloutStrategyInfo.allAtOnceStrategyInfo.clusters.endTime:label=END_TIME,
                    info.rolloutStrategyInfo.allAtOnceStrategyInfo.clusters.state:label=STATE,
                    all_messages():label=MESSAGES)"""


def ApiVersion():
  return _DEFAULT_API_VERSION


def FormatForRolloutsDescribe(rollout, args, less=False):
  """Sets format for `rollouts describe` depending on rollout strategy.

  Args:
    rollout: Rollout from `rollouts describe`
    args: Command line args
    less: Whether to show truncate rollout messages

  Returns:
    None
  """
  if rollout is None:
    return
  if rollout.info and rollout.info.rolloutStrategyInfo:
    if rollout.info.rolloutStrategyInfo.rollingStrategyInfo:
      if less:
        args.format = ROLLOUTS_DESCRIBE_ROLLING_TRUNCATED_MESSAGES_FORMAT
      else:
        args.format = ROLLOUTS_DESCRIBE_ROLLING_FULL_MESSAGES_FORMAT
      args.flatten = ['info.rolloutStrategyInfo.rollingStrategyInfo.clusters[]']
    if rollout.info.rolloutStrategyInfo.allAtOnceStrategyInfo:
      if less:
        args.format = ROLLOUTS_DESCRIBE_ALLATONCE_TRUNCATED_MESSAGES_FORMAT
      else:
        args.format = ROLLOUTS_DESCRIBE_ALLATONCE_FULL_MESSAGES_FORMAT
      args.flatten = [
          'info.rolloutStrategyInfo.allAtOnceStrategyInfo.clusters[]'
      ]


def _LoadResourcesFromFile(path):
  if not (path.endswith('.yaml') or path.endswith('.yml')):
    return []
  if os.path.isdir(path):
    return []
  resources = []
  loaded_resources = yaml.load_all_path(path)
  for resource in loaded_resources:
    if resource:
      dumped_resource = yaml.dump(resource)
      resources.append(dumped_resource)
  return resources


def _AllFilesUnderDir(path):
  """Returns paths of all .yaml and .yml files in a directory and subdirectories.

  Args:
    path: The root directory to search in.

  Returns:
    A list of strings, where each string is a path to a .yaml or .yml file.
  """
  if not path.endswith('/'):
    path += '/'
  glob_path = os.path.dirname(path) + '/**/*'
  return [
      p
      for p in glob.glob(pathname=glob_path, recursive=True)
      if os.path.isfile(p) and p.endswith(('.yaml', '.yml'))
  ]


def _VariantNameFromFilePath(path):
  return pathlib.Path(path).stem


def _VariantNameFromDir(path):
  return pathlib.Path(path).name


def ExpandPathForUser(path):
  return os.path.expanduser(path)


def _ExpandPathForUserAndVars(path):
  user_expanded_path = os.path.expanduser(path)
  vars_expanded_path = user_expanded_path
  if '$' in vars_expanded_path:
    vars_expanded_path = os.path.expandvars(vars_expanded_path)
  return vars_expanded_path


def _FileNotFoundMessage(path):
  return f'Source file or dir not found: {path}.'


def _FileWrongTypeMessage(path):
  return f'Source is not of type directory or file: {path}.'


def VariantsFromGlobPattern(glob_pattern):
  """Returns a dictionary of input-format variants discovered from a glob.

  Gets all files from user-supplied glob pattern and creates variant(s). If
  glob_pattern is for YAML files (e.g. *.yaml), variants are created from each
  file matched. If glob_pattern is for directories (e.g. *), variants are
  created from each directory matched. These variant(s) can be used for more
  advanced deployment setups.

  Args:
    glob_pattern: Pattern compatible with python's glob library

  Returns:
    A dict of input-formatted variants, for example:
      {'us-a': [resources...], 'us-b': [resources...]}

  Raises:
    ValueError: if duplicate variant names are detected.
  """
  user_expanded_glob = os.path.expanduser(glob_pattern)
  expanded_glob = user_expanded_glob
  if '$' in expanded_glob:
    expanded_glob = os.path.expandvars(expanded_glob)

  resources_by_variant_name = {}
  origin_by_variant_name = (
      {}
  )  # To track origins of variant names for dup detection.
  is_file_mask = '.yaml' in glob_pattern or '.yml' in glob_pattern
  paths = glob.glob(expanded_glob)

  if not is_file_mask:
    paths = [p for p in paths if os.path.isdir(p)]

  paths.sort()
  for path in paths:
    if is_file_mask:
      if os.path.isfile(path):
        resources = _LoadResourcesFromFile(path)
        if resources:
          variant_name = _VariantNameFromFilePath(path)
          if variant_name in origin_by_variant_name:
            raise ValueError(
                f'Duplicate variant name {variant_name!r} detected: paths'
                f' {origin_by_variant_name[variant_name]!r} and {path!r}'
                ' resulted in the same variant name.'
            )
          resources_by_variant_name[variant_name] = resources
          origin_by_variant_name[variant_name] = path
    else:  # directory mask
      if os.path.isdir(path):
        files_list = _AllFilesUnderDir(path)
        resources_per_file = (
            _LoadResourcesFromFile(os.path.abspath(file))
            for file in files_list
        )
        all_resources = list(itertools.chain.from_iterable(resources_per_file))
        if all_resources:
          variant_name = _VariantNameFromDir(path)
          if variant_name in origin_by_variant_name:
            raise ValueError(
                f'Duplicate variant name {variant_name!r} detected: paths'
                f' {origin_by_variant_name[variant_name]!r} and {path!r}'
                ' resulted in the same variant name.'
            )
          resources_by_variant_name[variant_name] = all_resources
          origin_by_variant_name[variant_name] = path
  return resources_by_variant_name


def TransformTrimClusterLevelMessages(resource):
  """Shows the first cluster-level message and truncates it if it's too long.

  Args:
    resource: A RolloutInfo resource

  Returns:
    Message limited to 40 characters
  """
  truncated_message_length = 40
  messages = _GetClusterLevelMessagesFromResource(resource)
  if not messages:
    return ''
  if len(messages) >= 1 and len(messages[0]) > truncated_message_length:
    return messages[0][:truncated_message_length] + '...'
  return messages[0]


def TransformTrimRolloutLevelMessage(resource):
  """Trims rollout-level message if it's too long.

  Args:
    resource: A Rollout resource

  Returns:
    String message limited to 40 characters
  """
  rollout_info = resource.get('info', {})
  if rollout_info:
    rollout_message = rollout_info.get('message', '')
    if rollout_message:
      if len(rollout_message) > 40:
        return rollout_message[:40] + '...'
      return rollout_message
  return ''


def _GetClusterLevelMessagesFromResource(resource):
  """Gathers cluster-level messages from a Rollout resource.

  Args:
    resource: A Rollout resource, from `... rollouts describe ...`

  Returns:
    A list of messages from the Rollout resource.
  """
  messages = []
  if not resource:
    return []
  rollout_strategy_info = resource.get('info', {}).get(
      'rolloutStrategyInfo', {}
  )
  for rollout_info in rollout_strategy_info.values():
    clusters = rollout_info.get('clusters', [])
    if 'messages' in clusters:
      messages.extend(clusters.get('messages', []))
    current = clusters.get('current', {})
    if 'messages' in current:
      messages.extend(current.get('messages', []))

  info_errors = resource.get('info', {}).get('errors', [])
  if info_errors:
    for error in info_errors:
      info_message = error.get('errorMessage', '')
      if info_message:
        messages.append(info_message)

  return messages


def TransformAllClusterLevelMessages(resource):
  """Returns all cluster-level messages from a Rollout resource.

  Args:
    resource: A Rollout resource, from `... rollouts describe ...`

  Returns:
    A single string or string array of cluster-level messages.
  """
  messages = _GetClusterLevelMessagesFromResource(resource)
  if not messages:
    return ''
  elif len(messages) == 1:
    return messages[0]
  return messages


def TransformListFleetPackageErrors(resource):
  """Gathers errors from 'info.Errors' and returns their errorMessages."""
  messages = []
  if resource is None:
    return ''

  errors = resource.get('info', {}).get('errors', [])
  for error in errors:
    error_message = error.get('errorMessage', '')
    if error_message:
      messages.append(error_message)

  if not messages:
    return ''
  elif len(messages) == 1:
    return messages[0]
  return messages


def UpsertFleetPackageName(fleet_package, fully_qualified_name):
  """Upserts the correct fleet package name into fleet package resource.

  Args:
    fleet_package: A user-inputted FleetPackage which may or may not have a name
    fully_qualified_name: The fully qualified name of the FleetPackage resource.

  Returns:
    A FleetPackage that definitely has the correct fully qualified name.
  """
  if not fleet_package.name:
    fleet_package.name = fully_qualified_name
  return fleet_package


def FixFleetPackagePathForCloudBuild(fleet_package):
  """Removes leading slash from fleet package path if it uses Cloud Build.

  If we don't remove the leading slash, parsing the path will fail for cloud
  build. See b/352756986#comment13

  Args:
    fleet_package: A user-inputted FleetPackage which may need its path fixed.

  Returns:
    A FleetPackage with a fixed path if it uses Cloud Build, unchanged if it
    doesn't use Cloud Build.
  """
  if _FleetPackageUsesCloudBuild(fleet_package):
    path = fleet_package.resourceBundleSelector.cloudBuildRepository.path
    if path is not None and path.startswith('/'):
      if path == '/':
        fleet_package.resourceBundleSelector.cloudBuildRepository.path = './'
      else:
        fleet_package.resourceBundleSelector.cloudBuildRepository.path = (
            fleet_package.resourceBundleSelector.cloudBuildRepository.path[1:]
        )
  return fleet_package


def _FleetPackageUsesCloudBuild(fleet_package):
  return (
      fleet_package
      and fleet_package.resourceBundleSelector
      and fleet_package.resourceBundleSelector.cloudBuildRepository is not None
  )



# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Fleet API utils."""


import re

from googlecloudsdk.api_lib.container.fleet import types
from googlecloudsdk.api_lib.util import apis
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import parser_extensions
from googlecloudsdk.core import exceptions
from googlecloudsdk.core import resources
from googlecloudsdk.generated_clients.apis.gkehub.v1alpha import gkehub_v1alpha_messages as alpha_messages


VERSION_MAP = {
    base.ReleaseTrack.ALPHA: 'v1alpha',
    base.ReleaseTrack.BETA: 'v1beta',
    base.ReleaseTrack.GA: 'v1',
}

V2_VERSION_MAP = {
    base.ReleaseTrack.ALPHA: 'v2alpha',
    base.ReleaseTrack.BETA: 'v2beta',
    base.ReleaseTrack.GA: 'v2',
}


def GetMessagesModule(release_track=base.ReleaseTrack.GA):
  return apis.GetMessagesModule('gkehub', VERSION_MAP[release_track])


def GetV2MessagesModule(release_track=base.ReleaseTrack.ALPHA):
  return apis.GetMessagesModule('gkehub', V2_VERSION_MAP[release_track])


def FleetMessageModule(release_track: base.ReleaseTrack):
  """Dynamically load Fleet message module based on command track.

  Explicitly import message to enable type hint in Cider-V since
  `apis.GetMessagesModule()` cannot derive type to the specific Python module.

  Args:
    release_track: Determines the generated API message module to be returned.

  Returns:
    An API message module that corresponds to the release track.
  """
  if release_track == base.ReleaseTrack.ALPHA:
    return alpha_messages

  raise NotImplementedError(
      'Fleet command has not been promoted to {} track.'.format(
          release_track.name
      )
  )


def GetClientInstance(
    release_track=base.ReleaseTrack.GA,
) -> types.TrackClient:
  return apis.GetClientInstance('gkehub', VERSION_MAP[release_track])


def GetV2ClientInstance(
    release_track=base.ReleaseTrack.ALPHA,
) -> types.V2TrackClient:
  return apis.GetClientInstance('gkehub', V2_VERSION_MAP[release_track])


def GetClientClass(release_track=base.ReleaseTrack.GA):
  return apis.GetClientClass('gkehub', VERSION_MAP[release_track])


def LocationResourceName(project, location='global'):
  # See command_lib/container/fleet/resources.yaml
  return resources.REGISTRY.Create(
      'gkehub.projects.locations',
      projectsId=project,
      locationsId=location,
  ).RelativeName()


def MembershipLocation(full_name):
  matches = re.search('projects/.*/locations/(.*)/memberships/(.*)', full_name)
  if matches:
    return matches.group(1)
  raise exceptions.Error(
      'Invalid membership resource name: {}'.format(full_name)
  )


def MembershipResourceName(project, membership, location='global'):
  # See command_lib/container/fleet/resources.yaml
  return resources.REGISTRY.Create(
      'gkehub.projects.locations.memberships',
      projectsId=project,
      locationsId=location,
      membershipsId=membership,
  ).RelativeName()


def MembershipFeatureResourceName(
    project,
    membership,
    feature,
    location='global',
    release_track=base.ReleaseTrack.ALPHA,
):
  # See command_lib/container/fleet/resources.yaml
  return resources.REGISTRY.Parse(
      line=None,
      params={
          'projectsId': project,
          'locationsId': location,
          'membershipsId': membership,
          'featuresId': feature,
      },
      collection='gkehub.projects.locations.memberships.features',
      # Defaults to v1 without specifying api version.
      api_version=V2_VERSION_MAP[release_track],
  ).RelativeName()


def MembershipPartialName(full_name):
  matches = re.search('projects/.*/locations/(.*)/memberships/(.*)', full_name)
  if matches:
    return matches.group(1) + '/' + matches.group(2)
  raise exceptions.Error(
      'Invalid membership resource name: {}'.format(full_name)
  )


def MembershipShortname(full_name):
  return resources.REGISTRY.ParseRelativeName(
      full_name, collection='gkehub.projects.locations.memberships'
  ).Name()


def FeatureResourceName(project, feature, location='global'):
  # See command_lib/container/fleet/resources.yaml
  return resources.REGISTRY.Create(
      'gkehub.projects.locations.features',
      projectsId=project,
      locationsId=location,
      featuresId=feature,
  ).RelativeName()


def OperationResourceName(project, operation, location='global'):
  # See command_lib/container/fleet/resources.yaml
  return resources.REGISTRY.Create(
      'gkehub.projects.locations.operations',
      projectsId=project,
      locationsId=location,
      operationsId=operation,
  ).RelativeName()


def FleetRef(
    project,
    fleet='default',
    location='global',
    release_track=base.ReleaseTrack.ALPHA,
) -> resources.Resource:
  return resources.REGISTRY.Parse(
      line=None,
      params={
          'projectsId': project,
          'locationsId': location,
          'fleetsId': fleet,
      },
      collection='gkehub.projects.locations.fleets',
      api_version=VERSION_MAP[release_track],
  )


def FleetResourceName(
    project,
    fleet='default',
    location='global',
    release_track=base.ReleaseTrack.ALPHA,
) -> str:
  # See command_lib/container/fleet/resources.yaml
  return FleetRef(project, fleet, location, release_track).RelativeName()


def FleetParentName(
    project, location='global', release_track=base.ReleaseTrack.ALPHA
):
  # See command_lib/container/fleet/resources.yaml
  return resources.REGISTRY.Parse(
      line=None,
      params={
          'projectsId': project,
          'locationsId': location,
      },
      collection='gkehub.projects.locations',
      api_version=VERSION_MAP[release_track],
  ).RelativeName()


def FleetOrgParentName(organization, location='global'):
  return 'organizations/{0}/locations/{1}'.format(organization, location)


def ScopeParentName(project, release_track=base.ReleaseTrack.ALPHA):
  # See command_lib/container/fleet/resources.yaml
  return resources.REGISTRY.Parse(
      line=None,
      params={
          'projectsId': project,
          'locationsId': 'global',
      },
      collection='gkehub.projects.locations',
      api_version=VERSION_MAP[release_track],
  ).RelativeName()


def NamespaceParentName(project, release_track=base.ReleaseTrack.ALPHA):
  # See command_lib/container/fleet/resources.yaml
  return resources.REGISTRY.Parse(
      line=None,
      params={
          'projectsId': project,
          'locationsId': 'global',
      },
      collection='gkehub.projects.locations',
      api_version=VERSION_MAP[release_track],
  ).RelativeName()


def NamespaceResourceName(project, name, release_track=base.ReleaseTrack.ALPHA):
  # See command_lib/container/fleet/resources.yaml
  return resources.REGISTRY.Parse(
      line=None,
      params={
          'projectsId': project,
          'locationsId': 'global',
          'namespacesId': name,
      },
      collection='gkehub.projects.locations.namespaces',
      api_version=VERSION_MAP[release_track],
  ).RelativeName()


def ScopeNamespaceParentName(
    project, scope, release_track=base.ReleaseTrack.ALPHA
):
  # See command_lib/container/fleet/resources.yaml
  return resources.REGISTRY.Parse(
      line=None,
      params={
          'projectsId': project,
          'locationsId': 'global',
          'scopesId': scope,
      },
      collection='gkehub.projects.locations.scopes',
      api_version=VERSION_MAP[release_track],
  ).RelativeName()


def ScopeNamespaceResourceName(
    project, scope, name, release_track=base.ReleaseTrack.ALPHA
):
  # See command_lib/container/fleet/resources.yaml
  return resources.REGISTRY.Parse(
      line=None,
      params={
          'projectsId': project,
          'locationsId': 'global',
          'scopesId': scope,
          'namespacesId': name,
      },
      collection='gkehub.projects.locations.scopes.namespaces',
      api_version=VERSION_MAP[release_track],
  ).RelativeName()


def RBACRoleBindingParentName(
    project, namespace, release_track=base.ReleaseTrack.ALPHA
):
  # See command_lib/container/fleet/resources.yaml
  return resources.REGISTRY.Parse(
      line=None,
      params={
          'projectsId': project,
          'locationsId': 'global',
          'namespacesId': namespace,
      },
      collection='gkehub.projects.locations.namespaces',
      api_version=VERSION_MAP[release_track],
  ).RelativeName()


def ScopeRBACRoleBindingParentName(
    project, scope, release_track=base.ReleaseTrack.ALPHA
):
  # See command_lib/container/fleet/resources.yaml
  return resources.REGISTRY.Parse(
      line=None,
      params={
          'projectsId': project,
          'locationsId': 'global',
          'scopesId': scope,
      },
      collection='gkehub.projects.locations.scopes',
      api_version=VERSION_MAP[release_track],
  ).RelativeName()


def RBACRoleBindingResourceName(
    project, namespace, name, release_track=base.ReleaseTrack.ALPHA
):
  # See command_lib/container/fleet/resources.yaml
  return resources.REGISTRY.Parse(
      line=None,
      params={
          'projectsId': project,
          'locationsId': 'global',
          'namespacesId': namespace,
          'rbacrolebindingsId': name,
      },
      collection='gkehub.projects.locations.namespaces.rbacrolebindings',
      api_version=VERSION_MAP[release_track],
  ).RelativeName()


def ScopeRBACRoleBindingResourceName(
    project, scope, name, release_track=base.ReleaseTrack.ALPHA
):
  # See command_lib/container/fleet/resources.yaml
  return resources.REGISTRY.Parse(
      line=None,
      params={
          'projectsId': project,
          'locationsId': 'global',
          'scopesId': scope,
          'rbacrolebindingsId': name,
      },
      collection='gkehub.projects.locations.scopes.rbacrolebindings',
      api_version=VERSION_MAP[release_track],
  ).RelativeName()


def MembershipRBACRoleBindingResourceName(
    project, location, membership, name, release_track=base.ReleaseTrack.ALPHA
):
  """Parses a Membership RBAC Role Binding resource.

  Args:
    project: the full project ID or number for the resource.
    location: the location of the resource.
    membership: the parent membership of the resource.
    name: the resource name for the role binding.
    release_track: the API version for the resource parsing.

  Returns:
    A Membership RBAC Role Binding resource.
  """
  # See command_lib/container/fleet/resources.yaml
  return resources.REGISTRY.Parse(
      line=None,
      params={
          'projectsId': project,
          'locationsId': location,
          'membershipsId': membership,
          'rbacrolebindingsId': name,
      },
      collection='gkehub.projects.locations.memberships.rbacrolebindings',
      api_version=VERSION_MAP[release_track],
  ).RelativeName()


def MembershipBindingResourceName(
    project,
    name,
    membership,
    location='global',
    release_track=base.ReleaseTrack.GA,
):
  # See command_lib/container/fleet/resources.yaml
  return resources.REGISTRY.Parse(
      line=None,
      params={
          'projectsId': project,
          'locationsId': location,
          'membershipsId': membership,
          'bindingsId': name,
      },
      collection='gkehub.projects.locations.memberships.bindings',
      api_version=VERSION_MAP[release_track],
  ).RelativeName()


def MembershipBindingParentName(
    project, membership, location='global', release_track=base.ReleaseTrack.GA
):
  # See command_lib/container/fleet/resources.yaml
  return resources.REGISTRY.Parse(
      line=None,
      params={
          'projectsId': project,
          'locationsId': location,
          'membershipsId': membership,
      },
      collection='gkehub.projects.locations.memberships',
      api_version=VERSION_MAP[release_track],
  ).RelativeName()


def ScopeResourceName(project, scope, location='global'):
  # See command_lib/container/fleet/resources.yaml
  return resources.REGISTRY.Create(
      'gkehub.projects.locations.scopes',
      projectsId=project,
      locationsId=location,
      scopesId=scope,
  ).RelativeName()


def OperationRef(operation: alpha_messages.Operation) -> resources.Resource:
  """Parses a gkehub Operation reference from an operation."""
  return resources.REGISTRY.ParseRelativeName(
      operation.name, collection='gkehub.projects.locations.operations'
  )


def RolloutRef(args: parser_extensions.Namespace) -> resources.Resource:
  if getattr(args.CONCEPTS, 'rollout', None):
    return args.CONCEPTS.rollout.Parse()


def RolloutName(args: parser_extensions.Namespace) -> str:
  rollout_ref = RolloutRef(args)
  if rollout_ref:
    return rollout_ref.RelativeName()
  return None


def RolloutParentName(args: parser_extensions.Namespace):
  rollout_ref = RolloutRef(args)
  if rollout_ref:
    return rollout_ref.Parent().RelativeName()
  return None


def RolloutId(args: parser_extensions.Namespace) -> str:
  rollout_ref = RolloutRef(args)
  if rollout_ref:
    return rollout_ref.Name()
  return None


def RolloutSequenceRef(
    args: parser_extensions.Namespace,
) -> resources.Resource:
  if getattr(args.CONCEPTS, 'rolloutsequence', None):
    return args.CONCEPTS.rolloutsequence.Parse()
  return None


def RolloutSequenceName(args: parser_extensions.Namespace) -> str:
  rollout_sequence_ref = RolloutSequenceRef(args)
  if rollout_sequence_ref:
    return rollout_sequence_ref.RelativeName()
  return None


def RolloutSequenceParentName(args: parser_extensions.Namespace):
  rollout_sequence_ref = RolloutSequenceRef(args)
  if rollout_sequence_ref:
    return rollout_sequence_ref.Parent().RelativeName()
  return None


def RolloutSequenceId(args: parser_extensions.Namespace) -> str:
  rollout_sequence_ref = RolloutSequenceRef(args)
  if rollout_sequence_ref:
    return rollout_sequence_ref.Name()
  return None



# -*- coding: utf-8 -*- #
# Copyright 2014 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Command for deleting access configs from virtual machine instances."""


from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.compute import constants
from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute.instances import flags


class DeleteAccessConfig(base.SilentCommand):
  """Delete an access configuration from a virtual machine network interface."""

  detailed_help = {
      'DESCRIPTION': """
        *{command}* is used to delete access configurations from network
        interfaces of Compute Engine virtual machines. Access
        configurations let you assign a public, external IP to a virtual
        machine. The delete-access-config operation removes external IP from
        the instance interface. If there is traffic routed to the external IP,
        after deleting the access config operation, traffic to the external IP
        will not reach the VM anymore.
      """,
      'EXAMPLES': """
        To remove the externally accessible IP from a virtual machine named
        ``example-instance'' in zone ``us-central1-a'', run:

          $ {command} example-instance --zone=us-central1-a
      """,
  }

  @staticmethod
  def Args(parser):
    flags.INSTANCE_ARG.AddArgument(parser)
    parser.add_argument(
        '--access-config-name',
        default=constants.DEFAULT_ACCESS_CONFIG_NAME,
        help="""\
        Specifies the name of the access configuration to delete.
        ``{0}'' is used as the default if this flag is not provided.
        """.format(constants.DEFAULT_ACCESS_CONFIG_NAME),
    )
    parser.add_argument(
        '--network-interface',
        default=constants.DEFAULT_NETWORK_INTERFACE,
        action=arg_parsers.StoreOnceAction,
        help="""\
        Specifies the name of the network interface from which to delete the
        access configuration. If this is not provided, then ``nic0'' is used
        as the default.
        """,
    )

  def Run(self, args):
    """Invokes request necessary for removing an access config."""
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    client = holder.client

    instance_ref = flags.INSTANCE_ARG.ResolveAsResource(
        args,
        holder.resources,
        scope_lister=flags.GetInstanceZoneScopeLister(client),
    )

    request = client.messages.ComputeInstancesDeleteAccessConfigRequest(
        accessConfig=args.access_config_name,
        instance=instance_ref.Name(),
        networkInterface=args.network_interface,
        project=instance_ref.project,
        zone=instance_ref.zone,
    )

    return client.MakeRequests(
        [(client.apitools_client.instances, 'DeleteAccessConfig', request)]
    )



# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Command for listing GlobalVmExtensionPolicies."""

from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.calliope import base
from googlecloudsdk.core import properties


@base.DefaultUniverseOnly
@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA)
class List(base.ListCommand):
  """List Compute Engine global VM extension policies."""

  detailed_help = {
      'brief': 'List Compute Engine global VM extension policies.',
      'EXAMPLES': """
     To list all global VM extension policy, run:

       $ {command}
   """,
  }

  def Run(self, args):
    r"""Run the List command.

    Args:
      args: argparse.Namespace, The arguments to this command.

    Returns:
      Response calling the GlobalVmExtensionPoliciesService.List API.
    """
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    client = holder.client
    messages = holder.client.messages
    return client.MakeRequests([(
        client.apitools_client.globalVmExtensionPolicies,
        'List',
        messages.ComputeGlobalVmExtensionPoliciesListRequest(
            project=properties.VALUES.core.project.GetOrFail(),
        ),
    )])



# -*- coding: utf-8 -*- #
# Copyright 2026 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for getting the version of a reservation slot."""

from apitools.base.py import encoding
from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.compute.operations import poller
from googlecloudsdk.api_lib.util import waiter
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute import flags as compute_flags
from googlecloudsdk.command_lib.compute import scope as compute_scope
from googlecloudsdk.command_lib.compute.reservations import resource_args
from googlecloudsdk.command_lib.compute.reservations.slots import flags
from googlecloudsdk.core import exceptions as core_exceptions


class GetVersionPoller(poller.Poller):
  """Custom poller that returns the operation response directly.

  This avoids the AttributeError caused by the default Poller trying to
  re-fetch the resource using incompatible field names.
  """

  def __init__(self, resource_service, is_current):
    super(GetVersionPoller, self).__init__(resource_service)
    self.is_current = is_current

  def GetResult(self, operation):
    """Returns a nested dictionary for structured formatting."""
    metadata = getattr(operation, 'getVersionOperationMetadata', None)
    if not metadata:
      return None

    # Decode the metadata into a Python dictionary
    metadata_dict = encoding.MessageToPyValue(metadata)
    sbom_info = metadata_dict.get('inlineSbomInfo', {})

    # Return a nested dictionary using the component name maps directly
    if self.is_current:
      return {
          'currentComponentVersions': sbom_info.get(
              'currentComponentVersions', {}
          )
      }
    else:
      return {
          'targetComponentVersions': sbom_info.get(
              'targetComponentVersions', {}
          )
      }


@base.UniverseCompatible
@base.ReleaseTracks(
    base.ReleaseTrack.GA, base.ReleaseTrack.BETA, base.ReleaseTrack.ALPHA
)
class GetVersion(base.Command):
  """Get the version of a reservation slot."""

  @staticmethod
  def Args(parser):
    resource_args.GetReservationResourceArg().AddArgument(
        parser, operation_type='get-version'
    )
    flags.AddGetVersionFlags(parser)

  def Run(self, args):
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    client = holder.client
    messages = client.messages

    reservation_ref = resource_args.GetReservationResourceArg(
    ).ResolveAsResource(
        args,
        holder.resources,
        default_scope=compute_scope.ScopeEnum.ZONE,
        scope_lister=compute_flags.GetDefaultScopeLister(client))

    parent_name = f'reservations/{reservation_ref.reservation}/reservationBlocks/{args.block_name}/reservationSubBlocks/{args.sub_block_name}'

    sbom_selections_enum = (
        messages.ReservationSlotsGetVersionRequest.SbomSelectionsValueListEntryValuesEnum
    )
    sbom_selections = []
    if args.current:
      sbom_selections.append(sbom_selections_enum.SBOM_SELECTION_CURRENT)
    if args.target:
      sbom_selections.append(sbom_selections_enum.SBOM_SELECTION_TARGET)

    # Create the inner message instance for the request body
    body_message = messages.ReservationSlotsGetVersionRequest()
    if sbom_selections:
      body_message.sbomSelections = sbom_selections

    # Create the outer request instance with path/query parameters
    request = messages.ComputeReservationSlotsGetVersionRequest(
        parentName=parent_name,
        zone=reservation_ref.zone,
        project=reservation_ref.project,
        reservationSlot=args.slot_name,
        # Assign the inner message to the body field
        reservationSlotsGetVersionRequest=body_message
    )

    errors = []
    responses = client.AsyncRequests(
        [(client.apitools_client.reservationSlots, 'GetVersion', request)],
        errors_to_collect=errors
    )

    # Check for and raise any RPC errors
    if errors:
      raise core_exceptions.MultiError(errors)

    # Parse the response
    operation_refs = [holder.resources.Parse(r.selfLink) for r in responses]
    operation_ref = operation_refs[0]

    operation_poller = GetVersionPoller(
        holder.client.apitools_client.reservationSlots,
        is_current=args.current
    )
    result = waiter.WaitFor(
        operation_poller,
        operation_ref,
        'Waiting for operation [{}] to complete'.format(operation_ref.Name()),
    )
    return result


GetVersion.detailed_help = {
    'EXAMPLES': (
        """\
    To get the current version of a reservation slot in reservation
    exr-1 in ZONE with block name block-1, sub-block name sub-block-1
    and slot name slot-1, run:

      $ {command} exr-1 --zone=ZONE --block-name=block-1 \
          --sub-block-name=sub-block-1 --slot-name=slot-1 --current
    """
    ),
}



# -*- coding: utf-8 -*- #
# Copyright 2024 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Common command-agnostic utility functions for server-certs commands."""


ACTIVE_CERT_LABEL = 'Active'
NEXT_CERT_LABEL = 'Next'
PREVIOUS_CERT_LABEL = 'Previous'


def ListServerCertificates(sql_client, sql_messages, instance_ref):
  """Calls the list server certs endpoint and returns the response."""
  return sql_client.instances.ListServerCertificates(
      sql_messages.SqlInstancesListServerCertificatesRequest(
          project=instance_ref.project, instance=instance_ref.instance
      )
  )


def GetServerCertificateTypeDict(list_server_certs_response):
  """Gets a dictionary mapping Server Cert types to certs.

  The keys to the dictionary returned will be some combinatiaon of 'Current',
  'Next', and 'Previous'.

  Args:
    list_server_certs_response: InstancesListServerCertificatesResponse
      instance.

  Returns:
    A dictionary mapping Server Cert types to SslCert instances.
  """
  server_cert_types = {}

  active_id = list_server_certs_response.activeVersion

  # Get the active cert.
  certs = list_server_certs_response.serverCerts
  active_cert = None
  for cert in certs:
    if cert.sha1Fingerprint == active_id:
      active_cert = cert
      break
  if not active_cert:
    # No server cert types can be discerned; return an empty dict.
    return server_cert_types
  server_cert_types[ACTIVE_CERT_LABEL] = active_cert

  # Get the inactive certs.
  inactive_certs = [cert for cert in certs if cert.sha1Fingerprint != active_id]
  if len(inactive_certs) == 1:
    inactive_cert = inactive_certs[0]
    if inactive_cert.createTime > active_cert.createTime:
      # Found the next cert.
      server_cert_types[NEXT_CERT_LABEL] = inactive_cert
    else:
      # Found the previous cert.
      server_cert_types[PREVIOUS_CERT_LABEL] = inactive_cert
  elif len(inactive_certs) > 1:
    # Sort by expiration date.
    inactive_certs = sorted(inactive_certs, key=lambda cert: cert.createTime)
    server_cert_types[PREVIOUS_CERT_LABEL] = inactive_certs[0]
    server_cert_types[NEXT_CERT_LABEL] = inactive_certs[-1]

  return server_cert_types


def GetCurrentServerCertificate(sql_client, sql_messages, instance_ref):
  """Returns the currently active Server Cert."""
  server_cert_types = GetServerCertificateTypeDict(
      ListServerCertificates(sql_client, sql_messages, instance_ref)
  )
  return server_cert_types.get(ACTIVE_CERT_LABEL)


def GetNextServerCertificate(sql_client, sql_messages, instance_ref):
  """Returns the upcoming Server Cert."""
  server_cert_types = GetServerCertificateTypeDict(
      ListServerCertificates(sql_client, sql_messages, instance_ref)
  )
  return server_cert_types.get(NEXT_CERT_LABEL)


def GetPreviousServerCertificate(sql_client, sql_messages, instance_ref):
  """Returns the previously active Server Cert."""
  server_cert_types = GetServerCertificateTypeDict(
      ListServerCertificates(sql_client, sql_messages, instance_ref)
  )
  return server_cert_types.get(PREVIOUS_CERT_LABEL)



# -*- coding: utf-8 -*- #
# Copyright 2018 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for listing images."""


from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.containeranalysis import util as containeranalysis_util
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute.images.packages import exceptions
from googlecloudsdk.command_lib.compute.images.packages import filter_utils
from googlecloudsdk.command_lib.compute.images.packages import flags as package_flags
from googlecloudsdk.core import properties


class List(base.ListCommand):
  """List the packages in an image.
  """

  @staticmethod
  def Args(parser):
    parser.display_info.AddFormat("""\
        table(
          name:label=PACKAGE,
          version:label=VERSION,
          revision:label=REVISION
        )""")
    # Add resource flag for image.
    package_flags.AddImageResourceArg(parser)

  def _GetPackageVersions(self, image_packages, image_name):
    package_versions = []
    for occurrence in image_packages:
      package_name = occurrence.installation.name
      for location in occurrence.installation.location:
        package_version = {'name': package_name,
                           'version': location.version.name,
                           'revision': location.version.revision}
        package_versions.append(package_version)

    if not package_versions:
      raise exceptions.ImagePackagesInfoUnavailableException(image_name)

    return sorted(package_versions,
                  key=lambda package_version: package_version['name'])

  def Run(self, args):
    """Yields filtered packages."""
    project = properties.VALUES.core.project.Get()
    image_ref = args.CONCEPTS.image.Parse()

    # Use GA to construct the compute API holder since the containeranalysis
    # API always call compute v1 API to refer the compute resources.
    holder = base_classes.ComputeApiHolder(base.ReleaseTrack.GA)
    resource_filter = filter_utils.GetFilter(image_ref, holder)

    image_packages = containeranalysis_util.MakeOccurrenceRequest(
        project_id=project, resource_filter=resource_filter,
        occurrence_filter=None, resource_urls=None)

    return self._GetPackageVersions(image_packages, args.image)



# -*- coding: utf-8 -*- #
# Copyright 2024 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utilities for handling API endpoint overrides."""


from googlecloudsdk.api_lib.container.fleet import gkehub_api_util
from googlecloudsdk.api_lib.container.fleet.connectgateway import util as connectgateway_api_util
from googlecloudsdk.api_lib.util import apis as core_apis
from googlecloudsdk.core import exceptions
from googlecloudsdk.core import properties


class RegionalGatewayEndpoint:
  """Context manager for connecting to a particular regional Connect Gateway endpoint.

  This uses the provided region to temporarily override
  `api_endpoint_overrides/connectgateway` to a regional endpoint. If the
  `gkehub` endpoint is overridden, the `connectgateway` endpoint will use the
  same environment.

  This context manager is a no-op if the `connectgateway` endpoint is already
  overridden.
  """

  API_NAME = connectgateway_api_util.API_NAME
  API_VERSION = connectgateway_api_util.VERSION_MAP[
      connectgateway_api_util.DEFAULT_TRACK
  ]

  def __init__(self, region: str):
    """Initializes the context manager.

    Args:
      region: The Connect Gateway region to connect to.

    Raises:
      exceptions.Error: If `region` is Falsy.
    """

    if not region:
      raise exceptions.Error(
          'A region must be provided for the Gateway endpoint.'
      )
    self.region = region
    # The overridden endpoint value.
    self.endpoint: str = ''
    # The Connect Gateway endpoint override property.
    self.override = properties.VALUES.api_endpoint_overrides.Property(
        self.API_NAME
    )
    self._original_endpoint: str = ''

  def __enter__(self):
    try:
      hub_override = properties.VALUES.api_endpoint_overrides.Property(
          gkehub_api_util.GKEHUB_API_NAME
      ).Get()
      hub_version = gkehub_api_util.GKEHUB_GA_API_VERSION
    except properties.NoSuchPropertyError:
      hub_override = None
      hub_version = None

    if hub_override:
      subdomain_endpoint = core_apis.GetEffectiveApiEndpoint(
          gkehub_api_util.GKEHUB_API_NAME, hub_version
      )
    else:
      subdomain_endpoint = core_apis.GetEffectiveApiEndpoint(
          self.API_NAME, self.API_VERSION
      )

    if hub_override:
      subdomain_endpoint = hub_override.replace('gkehub', 'connectgateway')
    if self.region == 'global' or self.region in subdomain_endpoint:
      self.endpoint = subdomain_endpoint
    else:
      if subdomain_endpoint.startswith('https://'):
        self.endpoint = f'https://{self.region}-{subdomain_endpoint[8:]}'
      elif subdomain_endpoint.startswith('http://'):
        self.endpoint = f'http://{self.region}-{subdomain_endpoint[7:]}'
      else:
        raise exceptions.Error(
            f'Invalid Connect Gateway endpoint: {subdomain_endpoint}'
        )

    self._original_endpoint = self.override.Get()
    self.override.Set(self.endpoint)
    return self

  def __exit__(self, exc_type, exc_value, traceback):
    self.override.Set(self._original_endpoint)



# -*- coding: utf-8 -*- #
# Copyright 2014 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for listing persistent disks."""


from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.compute import lister
from googlecloudsdk.api_lib.compute import utils
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute import completers
from googlecloudsdk.command_lib.compute.disks import flags


@base.ReleaseTracks(
    base.ReleaseTrack.GA,
    base.ReleaseTrack.BETA,
    base.ReleaseTrack.ALPHA,
    base.ReleaseTrack.PREVIEW,
)
class List(base.ListCommand):
  """List Compute Engine persistent disks."""

  @staticmethod
  def Args(parser):
    parser.display_info.AddFormat(flags.MULTISCOPE_LIST_FORMAT)
    parser.display_info.AddUriFunc(utils.MakeGetUriFunc())
    lister.AddMultiScopeListerFlags(parser, zonal=True, regional=True)
    parser.display_info.AddCacheUpdater(completers.DisksCompleter)

  def Run(self, args):
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    client = holder.client

    request_data = lister.ParseMultiScopeFlags(args, holder.resources)

    list_implementation = lister.MultiScopeLister(
        client,
        zonal_service=client.apitools_client.disks,
        regional_service=client.apitools_client.regionDisks,
        aggregation_service=client.apitools_client.disks)

    return lister.Invoke(request_data, list_implementation)


List.detailed_help = base_classes.GetMultiScopeListerHelp(
    'disks',
    scopes=[
        base_classes.ScopeType.zonal_scope,
        base_classes.ScopeType.regional_scope
    ])
